==> pvfs-1.3.2.tgz: pvfs-1.3.2/mgr/meta/md_stat.c <==
/* md_stat.c copyright (c) 1996 Rob Ross and Matt Cettei, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: md_stat.c,v $
 * Revision 1.10  1998/08/07 17:29:02  spmcmil
 * Modified meta_access stuff and cleaned up.
 *
 * Revision 1.8  1998/07/21 04:22:59  rbross
 * *** empty log message ***
 *
 * Revision 1.7  1998/07/20 17:33:53  spmcmil
 * Modified to use meta functions (meta_write/read/creat/open/close/unlink).
 *
 * Revision 1.6  1998/02/25 21:24:54  walt
 * changed the interface to accept a string pointer rather than
 * a request pointer - allowed this call to be used by more than
 * one request type.
 *
 * Revision 1.5  1998/02/16 19:29:21  rbross
 * Newt patches
 *
 * Revision 1.4  1997/10/28 19:21:30  pvfs
 * added license
 *
 * Revision 1.3  1997/06/10 17:52:18  walt
 * updated directory handling code
 *
 * Revision 1.2  1997/01/29 20:00:00  rbross
 * modified to detect directories and return normal UNIX stat info
 *
 * Revision 1.1  1997/01/16 17:39:00  mcettei
 * Initial revision
 *
 *
/*/

static char md_stat_c[] = "$Header: /projects/pvfs/pvfs-working/mgr/meta/RCS/md_stat.c,v 1.10 1998/08/07 17:29:02 spmcmil Exp $";
/* This file contains the fstat metadata file call for the PVFS	*/
/* Also assumes space for request has been allocated 					*/

#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/param.h>
#include <linux/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <meta.h>
#include <req.h>

extern int errno;

int md_stat(request, metar_p)
mreq_p request;
fmeta_p metar_p;
{
	fmeta meta1;
	int fd, length;
	dmeta dir;
	char temp[MAXPATHLEN];

	/* Open metadata file */	
	if ((fd = meta_open(request->req.access.fname, O_RDONLY)) < 0) {
		perror("md_stat, meta_open");
		return (-1);
	}

	/* Read metadata file - read is done before access to determine if its a
	 * directory or not (if its directory we need to do execute permission
	 * checking on the entire directory and not just the parent)  */	
	if (meta_read(fd, &meta1) != sizeof(fmeta)) {
		if (errno != EISDIR) {
			perror("md_stat: meta_read");
			meta_close(fd); /* try to close */
			return(-1);
		}
		else /* directory - just return the normal stats */ {
			/* check entire directory path for execute permissions */
			/* we know its a directory, so setting 0 here inplace of fd allows
			 * meta_access to forgo that check */
			if (meta_access(0, request->req.access.fname, request->uid,
				request->gid, X_OK) < 0) {
				perror("md_stat: meta_access");
				meta_close(fd); /* try to close */
				return(-1);
			}
			/* get stats and fill in pvfs specific stuff */
			if (unix_fstat(fd, &(metar_p->u_stat)) < 0) {
				perror("md_stat: unix_fstat");
				meta_close(fd); /* try to close */
				return(-1);
			}
			/* Close metadata file */
			if (meta_close(fd) < 0) {
				perror("md_stat: meta_close");
				return(-1);
			}	
			if (get_dmeta(request->req.access.fname, &dir) < 0) {
				perror("md_stat: get_dmeta");
				return(-1);
			}
			metar_p->u_stat.st_mode = dir.dr_mode;
			metar_p->u_stat.st_uid = dir.dr_uid;
			metar_p->u_stat.st_gid = dir.dr_gid;
			return(0);
		}
	}

	/* ok, we know its a file */
	/* check execute permission on parent directory */
	strncpy(temp, request->req.access.fname, MAXPATHLEN);
	length = get_parent(temp); /* strip off filename */
	/* if length<0, CWD is being used and directory permissions will
	 * not be checked pursuant to the UNIX method */
	if (length >= 0) {
		/* check for permissions to read from directory */
		if (meta_access(0, temp, request->uid, request->gid, X_OK)
			< 0) {
			perror("md_stat: meta_access");
			meta_close(fd); /* try to close */
			return(-1);
		}
	}

	/* Close metadata file */
	if (meta_close(fd) < 0) {
		perror("md_stat: meta_close");
		return(-1);
	}	
	/* Do acknowledge and return */
	*metar_p = meta1;
	return (0);
}

==> pvfs-1.4.2.tgz: pvfs-1.4.2/mgr/meta/md_stat.c <==
/* md_stat.c copyright (c) 1996 Rob Ross and Matt Cettei, all rights reserved.
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
 * 
 */

static char md_stat_c[] = "$Header: /projects/cvsroot/pvfs/mgr/meta/md_stat.c,v 1.3 1999/10/15 22:06:46 rbross Exp $";
/* This file contains the fstat metadata file call for the PVFS	*/
/* Also assumes space for request has been allocated 					*/

#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/param.h>
#include <linux/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <meta.h>
#include <req.h>

extern int errno;

int md_stat(mreq_p request, fmeta_p metar_p)
{
	fmeta meta1;
	int fd, length;
	dmeta dir;
	char temp[MAXPATHLEN];

	/* zero the metadata */
	memset(metar_p, 0, sizeof(fmeta));

	/* Open metadata file */	
	if ((fd = meta_open(request->req.access.fname, O_RDONLY)) < 0) {
		perror("md_stat, meta_open");
		return (-1);
	}

	/* Read metadata file - read is done before access to determine if its a
	 * directory or not (if its directory we need to do execute permission
	 * checking on the entire directory and not just the parent)  */	
	if (meta_read(fd, &meta1) != sizeof(fmeta)) {
		if (errno != EISDIR) {
			perror("md_stat: meta_read");
			meta_close(fd); /* try to close */
			return(-1);
		}
		else /* directory - just return the normal stats */ {
			/* check entire directory path for execute permissions */
			/* we know its a directory, so setting 0 here inplace of fd allows
			 * meta_access to forgo that check */
			if (meta_access(0, request->req.access.fname, request->uid,
				request->gid, X_OK) < 0) {
				perror("md_stat: meta_access");
				meta_close(fd); /* try to close */
				return(-1);
			}
			/* get stats and fill in pvfs specific stuff */
			if (unix_fstat(fd, &(metar_p->u_stat)) < 0) {
				perror("md_stat: unix_fstat");
				meta_close(fd); /* try to close */
				return(-1);
			}
			/* Close metadata file */
			if (meta_close(fd) < 0) {
				perror("md_stat: meta_close");
				return(-1);
			}	
			if (get_dmeta(request->req.access.fname, &dir) < 0) {
				perror("md_stat: get_dmeta");
				return(-1);
			}
			metar_p->u_stat.st_mode = dir.dr_mode;
			metar_p->u_stat.st_uid = dir.dr_uid;
			metar_p->u_stat.st_gid = dir.dr_gid;
			return(0);
		}
	}

	/* ok, we know its a file */
	/* check execute permission on parent directory */
	strncpy(temp, request->req.access.fname, MAXPATHLEN);
	length = get_parent(temp); /* strip off filename */
	/* if length<0, CWD is being used and directory permissions will
	 * not be checked pursuant to the UNIX method */
	if (length >= 0) {
		/* check for permissions to read from directory */
		if (meta_access(0, temp, request->uid, request->gid, X_OK) < 0) {
			perror("md_stat: meta_access");
			meta_close(fd); /* try to close */
			return(-1);
		}
	}

	/* Close metadata file */
	if (meta_close(fd) < 0) {
		perror("md_stat: meta_close");
		return(-1);
	}	
	/* Do acknowledge and return */
	*metar_p = meta1;
	return (0);
}
